"""
Basics of whoosh:

- Create a schema
- Create an index
- Search the index
"""

# https://whoosh.readthedocs.io/en/latest/quickstart.html

###########################
# Create a schema
# Create an index

from whoosh.fields import Schema, STORED, ID, KEYWORD, TEXT

# designing a schema:
# https://whoosh.readthedocs.io/en/latest/schema.html

# create schema:
schema = Schema(title=TEXT(stored=True), 
                content=TEXT,
                path=ID(stored=True),
                tags=KEYWORD,
                icon=STORED)

import os.path
from whoosh.index import create_in

# create an index:
if not os.path.exists("index"):
        os.mkdir("index")
        ix = create_in("index", schema)


from whoosh.index import open_dir

# now open the index:
ix = open_dir("index")


###########################
# Write to the index

writer = ix.writer()

writer.add_document(title=u"My document",
                    content=u"This is my document!",
                    path=u"/a", 
                    tags=u"first short", 
                    icon=u"/icons/star.png")

writer.add_document(title=u"Second try",
                    content=u"This is the second example.",
                    path=u"/b", 
                    tags=u"second short", 
                    icon=u"/icons/sheep.png")

writer.add_document(title=u"Third time's the charm",
                    content=u"Examples are many.",
                    path=u"/c", 
                    tags=u"short", 
                    icon=u"/icons/book.png")

# the commit() call saves the added documents to the index:
writer.commit()



###########################
# Search the index


searcher = ix.searcher()

with ix.searcher() as searcher:

    # use the with block
    # so search is automatically
    # closed when finished

    # searcher.search() takes a Query objet
    # we should construct a query object DIRECTLY
    # or use a query parser to parse a query string

    # example: construct query string directly:
    # to match docs containing both apple and bear:
    # 
    # > from whoosh.query import *
    # > myquery = And([Term("content", u"apple"), Term("content", "bear")])

    # example: parse query string
    # To parse a query string, you can use the default 
    # query parser in the qparser module.
    # Parse a query string
    #
    # > from whoosh.qparser import QueryParser
    # > parser = QueryParser("content", ix.schema)
    # > myquery = parser.parse(querystring)

    
    # once you haave a searcher object,
    # call search() on it:
    # 
    # > results = searcher.search(myquery)
    # > print(len(results))
    # > print(results[0])


    # AND OR NOT
    # 
    # >>> print(parser.parse(u"render shade animate"))
    # And([Term("content", "render"), Term("content", "shade"), Term("content", "animate")])

    # >>> print(parser.parse(u"render OR (title:shade keyword:animate)"))
    # Or([Term("content", "render"), And([Term("title", "shade"), Term("keyword", "animate")])])

    # >>> print(parser.parse(u"rend*"))
    # Prefix("content", "rend")

    pass

